#pragma once

#include <cstdint>

#include <vector>

class Fader
{
	public:
		struct Color {
			int16_t r, g, b, w;

			Color(int16_t ir = 0, int16_t ig = 0, int16_t ib = 0, int16_t iw = 0) : r(ir), g(ig), b(ib), w(iw) {}

			void operator += (const Color &color)
			{
				this->r += color.r;
				this->g += color.g;
				this->b += color.b;
				this->w += color.w;
			}

			void normalize(void)
			{
				if(r > 255) { r = 255; } else if (r < 0) { r = 0; };
				if(g > 255) { g = 255; } else if (g < 0) { g = 0; };
				if(b > 255) { b = 255; } else if (b < 0) { b = 0; };
				if(w > 255) { w = 255; } else if (w < 0) { w = 0; };
			}

			void gamma_correct(void)
			{
				r = r * r / 255;
				g = g * g / 255;
				b = b * b / 255;
				w = w * w / 255;
			}
		};


		Fader(std::size_t nstrips, std::size_t nmodules_per_strip);

		void set_color(uint32_t strip, uint32_t module, const Color &color);
		void fade_color(uint32_t strip, uint32_t module, const Color &color);
		void add_color(uint32_t strip, uint32_t module, const Color &color);
		void set_color(const Color &color); // for all LEDs
		void fade_color(const Color &color); // for all LEDs
		void add_color(const Color &color); // for all LEDs
		void set_fadestep(uint8_t newFadestep);

		void update(void);

		/*!
		 * \returns Whether any color value has changed since the last call to getColorValues().
		 */
		bool something_changed(void) const {return !m_commands.empty(); };

		std::size_t modules_per_strip(void) { return m_modulesPerStrip; }
		std::size_t strips(void) { return m_strips; }

	private:
		enum Action {
			SET_COLOUR = 0,
			FADE_COLOUR = 1,
			ADD_COLOUR = 2,
			SET_FADESTEP = 3
		};

		struct Command {
			enum Action action;
			uint8_t     strip;
			uint8_t     module;
			Color       color;
		};

		int m_fd;

		std::size_t m_strips;
		std::size_t m_modulesPerStrip;

		std::vector<Command> m_commands;
};
